/*
 * This file is part of ascad.
 *
 * Ascad is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Ascad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ascad; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
/* Copyright (C) 2005 Andrei Krger */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>
#include "ascad.h"

/*****************************
 ***** Write Encoder *********
 *****************************
 */

static int A85Encode(encoder *ec, byte* data, long cc);
static int A85Flush(encoder *ec);

typedef struct {
  encoder self;
  FILE *out;
  dword tuple;
  byte state;
  int len;
} ascii85;

#define A85_LEN 72

void _a85encode(ascii85 *a85) {

  if (a85->tuple == 0) {
    fputc('z', a85->out);
    if (a85->len++ >= A85_LEN) { a85->len = 0; fputc('\n', a85->out); }
  } else {
    int i;
    char buf[5];

    for(i=0; i<5; i++) {
      buf[4-i] = a85->tuple % 85;
      a85->tuple /= 85;
    }
    for(i=0; i <= a85->state; i++) {
      fputc(buf[i] + '!', a85->out);
      if (a85->len++ >= A85_LEN) { a85->len = 0; fputc('\n', a85->out); }
    }
  }
  a85->tuple = a85->state = 0;
}

encoder *
InitA85Encoder(FILE *out)
{
  ascii85 *a85;

  if ((a85 = malloc(sizeof(ascii85))) == NULL)
    xerror("No space for A85 encoder");

  a85->self.encode = A85Encode;
  a85->self.flush = A85Flush;
  a85->out = out;
  a85->tuple = 0;
  a85->state = 0;
  a85->len = 0;
  return (encoder*)a85;
}

static int A85Encode(encoder *ec, byte* data, long cc) {
  ascii85 *a85 = (ascii85*)ec;
  while (cc--) {
    switch (a85->state++) {
      case 0: a85->tuple |= ((dword)*data++ << 24); break;
      case 1: a85->tuple |= ((dword)*data++ << 16); break;
      case 2: a85->tuple |= ((dword)*data++ <<  8); break;
      case 3: a85->tuple |= (dword)*data++;
              _a85encode(a85); break;
    }
  }
  return 0;
}

static int A85Flush(encoder *ec) {
  ascii85 *a85 = (ascii85*)ec;

  if (a85->state > 0) _a85encode(a85);
  if (a85->len + 2 > A85_LEN) fputc('\n', a85->out);
  a85->len = 0;
  fprintf(a85->out, "~>\n");
  return 0;
}

