/*
 * This file is part of ascad.
 *
 * Ascad is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Ascad is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ascad; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
/* Copyright (C) 2005 Andrei Krger */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <zlib.h>
#include "ascad.h"

/*****************************
 ***** Write Encoder *********
 *****************************
 */

static int GZEncode(encoder *ec, byte* data, long cc);
static int GZFlush(encoder *ec);
#define BUF_SIZE 8192

typedef struct {
  encoder self;
  encoder *next_encoder;
  byte buf[BUF_SIZE];
  z_stream strm;
} gz_encoder;

encoder *
InitGZEncoder(encoder *ec)
{
  gz_encoder *gz;
  int err;

  if ((gz = malloc(sizeof(gz_encoder))) == NULL)
    xerror("No space for GZ encoder");

  gz->self.encode = GZEncode;
  gz->self.flush = GZFlush;
  gz->next_encoder = ec;
  gz->strm.zalloc = (alloc_func)NULL;
  gz->strm.zfree = (free_func)NULL;
  if ((err = deflateInit(&gz->strm,  Z_DEFAULT_COMPRESSION)) != Z_OK)
    xerror("deflateInit: %s", zError(err));
  gz->strm.next_out = gz->buf;
  gz->strm.avail_out = BUF_SIZE;
  return (encoder*)gz;
}

static int GZEncode(encoder *ec, byte* data, long cc) {
  gz_encoder *gz = (gz_encoder*)ec;
  int err;

  gz->strm.next_in = data;
  gz->strm.avail_in = cc;
  do {
    // if ((err = deflate(&gz->strm, Z_NO_FLUSH)) != Z_OK)
    if ((err = deflate(&gz->strm, Z_SYNC_FLUSH)) != Z_OK)
      xerror("defalte: %s", zError(err));
    gz->next_encoder->encode(gz->next_encoder, gz->buf, BUF_SIZE - gz->strm.avail_out);
    gz->strm.next_out = gz->buf;
    gz->strm.avail_out = BUF_SIZE;
  } while (gz->strm.avail_in > 0);

  return 0;
}

static int GZFlush(encoder *ec) {
  gz_encoder *gz = (gz_encoder*)ec;
  int err;

  gz->strm.avail_in = 0;
  do {
    if ((err = deflate(&gz->strm,  Z_FINISH)) != Z_OK && err != Z_STREAM_END)
      xerror("deflate(FINISH): %s", zError(err));
    gz->next_encoder->encode(gz->next_encoder, gz->buf, BUF_SIZE - gz->strm.avail_out);
    gz->strm.next_out = gz->buf;
    gz->strm.avail_out = BUF_SIZE;
  } while (err != Z_STREAM_END);

  if ((err = deflateEnd(&gz->strm)) != Z_OK)
    xerror("deflateEnd: %s", zError(err));

  gz->next_encoder->flush(gz->next_encoder);

  return 0;
}

